import { Variables, utils, Events, Xml } from "blockly";
import * as blockly from "blockly";
import { sortWith, tryFind as tryFind$$1, head, find, last, iterate } from "./fable-library.2.4.11/Seq";
import { join } from "./fable-library.2.4.11/String";
import { replace } from "./fable-library.2.4.11/RegExp";
import { anonRecord, declare, Record } from "./fable-library.2.4.11/Types";
import { type, lambda, array as array$$6, record, bool, string } from "./fable-library.2.4.11/Reflection";
import { comparePrimitives, addToDict, tryGetValue, equals, createAtom } from "./fable-library.2.4.11/Util";
import { PromiseBuilder$$Run$$212F1D4B as PromiseBuilder$0024$0024Run$0024$0024212F1D4B, PromiseBuilder$$Delay$$62FBFDE1 as PromiseBuilder$0024$0024Delay$0024$002462FBFDE1, reject } from "./Fable.Promise.2.0.0/Promise";
import { promise } from "./Fable.Promise.2.0.0/PromiseImpl";
import * as rendermime from "@jupyterlab/rendermime";
import { addRangeInPlace, tryFind, zip, map } from "./fable-library.2.4.11/Array";

blockly.Python.finish = function (code) {
  const imports = [];
  const functions = [];
  const inputSequence = Object.keys(blockly.Python.definitions_);
  iterate(function (name) {
    const definitions = blockly.Python.definitions_;
    let def;
    const value = definitions[name];
    def = String(value);

    if (def.indexOf("import") >= 0) {
      imports.push(def);
    }

    if (def.indexOf("def ") === 0 ? true : def.indexOf("# ") === 0) {
      functions.push(def);
    }
  }, inputSequence);
  delete blockly.Python.definitions_;
  delete blockly.Python.functionNames_;
  blockly.Python.variableDB_.reset();
  return (join("\n", imports)) + (join("\n", functions)) + "\n\n" + code;
};

export function encodeWorkspace() {
  const xml = Xml.workspaceToDom(blockly.getMainWorkspace());
  const xmlText = Xml.domToText(xml);
  return xmlText;
}
export function decodeWorkspace(xmlText$$1) {
  const xml$$1 = Xml.textToDom(xmlText$$1);
  const value$$1 = Xml.domToWorkspace(xml$$1, blockly.getMainWorkspace());
  value$$1, null;
}
blockly.Blocks["comprehensionForEach_Python"] = {
  init() {
    console.log("comprehensionForEach_Python init");
    const value$$2 = this.appendValueInput("LIST").setCheck(null).appendField("for each item").appendField(new blockly.FieldVariable("i"), "VAR").appendField("in list");
    value$$2, null;
    const value$$3 = this.appendValueInput("YIELD").setCheck(null).setAlign(blockly.ALIGN_RIGHT).appendField("yield");
    value$$3, null;
    this.setOutput(true, null);
    this.setColour(230);
    this.setTooltip("Use this to generate a sequence of elements, also known as a comprehension. Often used for list comprehensions.");
    this.setHelpUrl("https://docs.python.org/3/tutorial/datastructures.html#list-comprehensions");
  }

};

blockly.Python["comprehensionForEach_Python"] = function (block) {
  var value$$4;
  const var$ = blockly.Python.variableDB_.getName((value$$4 = block.getFieldValue("VAR"), (String(value$$4))), blockly.Variables.NAME_TYPE);
  const list = blockly.Python.valueToCode(block, "LIST", blockly.Python.ORDER_ATOMIC);
  const yieldValue = blockly.Python.valueToCode(block, "YIELD", blockly.Python.ORDER_ATOMIC);
  const code$$1 = yieldValue + " for " + var$ + " in " + list;
  return [code$$1, blockly.Python.ORDER_ATOMIC];
};

blockly.Blocks["withAs_Python"] = {
  init() {
    console.log("withAs_Python init");
    const value$$5 = this.appendValueInput("EXPRESSION").setCheck(null).appendField("with");
    value$$5, null;
    const value$$6 = this.appendDummyInput().appendField("as").appendField(new blockly.FieldVariable("item"), "TARGET");
    value$$6, null;
    const value$$7 = this.appendStatementInput("SUITE").setCheck(null);
    value$$7, null;
    this.setNextStatement(true);
    this.setPreviousStatement(true);
    const value$$8 = this.setInputsInline(true);
    null, null;
    this.setColour(230);
    this.setTooltip("Use this to open resources (usually file-type) in a way that automatically handles errors and disposes of them when done. May not be supported by all libraries.");
    this.setHelpUrl("https://docs.python.org/3/reference/compound_stmts.html#with");
  }

};

blockly.Python["withAs_Python"] = function (block$$1) {
  var value$$10, copyOfStruct;
  let expressionCode;
  const value$$9 = blockly.Python.valueToCode(block$$1, "EXPRESSION", blockly.Python.ORDER_ATOMIC);
  expressionCode = String(value$$9);
  let targetCode;
  const value$$11 = blockly.Python.variableDB_.getName((value$$10 = block$$1.getFieldValue("TARGET"), (String(value$$10))), blockly.Variables.NAME_TYPE);
  targetCode = String(value$$11);
  const suiteCode = blockly.Python.statementToCode(block$$1, "SUITE");
  const code$$2 = "with " + expressionCode + " as " + targetCode + ":\n" + (copyOfStruct = suiteCode, String(copyOfStruct));
  return code$$2;
};

blockly.Blocks["textFromFile_Python"] = {
  init() {
    console.log("textFromFile_Python init");
    const value$$12 = this.appendValueInput("FILENAME").setCheck("String").appendField("read text from file");
    value$$12, null;
    this.setOutput(true, null);
    this.setColour(230);
    this.setTooltip("Use this to read a text file. It will output a string.");
    this.setHelpUrl("https://docs.python.org/3/tutorial/inputoutput.html");
  }

};

blockly.Python["textFromFile_Python"] = function (block$$2) {
  const fileName = blockly.Python.valueToCode(block$$2, "FILENAME", blockly.Python.ORDER_ATOMIC);
  const code$$3 = "open(" + fileName + ",encoding='utf-8').read()";
  return [code$$3, blockly.Python.ORDER_FUNCTION_CALL];
};

blockly.Blocks["readFile_Python"] = {
  init() {
    console.log("readFile_Python init");
    const value$$13 = this.appendValueInput("FILENAME").setCheck("String").appendField("read file");
    value$$13, null;
    this.setOutput(true, null);
    this.setColour(230);
    this.setTooltip("Use this to read a file. It will output a file, not a string.");
    this.setHelpUrl("https://docs.python.org/3/tutorial/inputoutput.html");
  }

};

blockly.Python["readFile_Python"] = function (block$$3) {
  const fileName$$1 = blockly.Python.valueToCode(block$$3, "FILENAME", blockly.Python.ORDER_ATOMIC);
  const code$$4 = "open('" + fileName$$1 + "',encoding='utf-8')";
  return [code$$4, blockly.Python.ORDER_FUNCTION_CALL];
};

export function makeCodeBlock_Python(blockName, hasInput, hasOutput) {
  blockly.Blocks[blockName] = {
    init() {
      const input = hasInput ? this.appendValueInput("INPUT").setCheck(null) : this.appendDummyInput();
      console.log(blockName + " init");
      const value$$14 = input.appendField(new blockly.FieldTextInput("type code here..."), "CODE");
      value$$14, null;

      if (hasOutput) {
        this.setOutput(true, null);
      } else {
        this.setNextStatement(true);
        this.setPreviousStatement(true);
      }

      this.setColour(230);
      this.setTooltip("You can put any Python code in this block. Use this block if you " + (hasInput ? "do" : "don't") + " need to connect an input block and " + (hasOutput ? "do" : "don't") + " need to connect an output block.");
      this.setHelpUrl("https://docs.python.org/3/");
    }

  };

  blockly.Python[blockName] = function (block$$4) {
    let userCode;
    const value$$15 = block$$4.getFieldValue("CODE");
    userCode = String(value$$15);
    let code$$5;

    if (hasInput) {
      const input$$1 = blockly.Python.valueToCode(block$$4, "INPUT", blockly.Python.ORDER_ATOMIC);
      code$$5 = (userCode + " " + input$$1).trim();
    } else {
      code$$5 = userCode.trim();
    }

    if (hasOutput) {
      return [code$$5, blockly.Python.ORDER_ATOMIC];
    } else {
      const value$$16 = code$$5 + "\n";
      return value$$16;
    }
  };
}
makeCodeBlock_Python("dummyOutputCodeBlock_Python", false, true);
makeCodeBlock_Python("dummyNoOutputCodeBlock_Python", false, false);
makeCodeBlock_Python("valueOutputCodeBlock_Python", true, true);
makeCodeBlock_Python("valueNoOutputCodeBlock_Python", true, false);
export function makeImportBlock_Python(blockName$$1, labelOne, labelTwo) {
  blockly.Blocks[blockName$$1] = {
    init() {
      const value$$17 = this.appendDummyInput().appendField(labelOne).appendField(new blockly.FieldTextInput("some library"), "libraryName").appendField(labelTwo).appendField(new blockly.FieldVariable("variable name"), "libraryAlias");
      value$$17, null;
      this.setNextStatement(true);
      this.setPreviousStatement(true);
      this.setColour(230);
      this.setTooltip("Import a python package to access functions in that package");
      this.setHelpUrl("https://docs.python.org/3/reference/import.html");
    }

  };

  blockly.Python[blockName$$1] = function (block$$5) {
    var value$$19;
    let libraryName;
    const value$$18 = block$$5.getFieldValue("libraryName");
    libraryName = String(value$$18);
    const libraryAlias = blockly.Python.variableDB_.getName((value$$19 = block$$5.getFieldValue("libraryAlias"), (String(value$$19))), blockly.Variables.NAME_TYPE);
    const code$$6 = labelOne + " " + libraryName + " " + labelTwo + " " + libraryAlias + "\n";
    return code$$6;
  };
}
makeImportBlock_Python("importAs_Python", "import", "as");
makeImportBlock_Python("importFrom_Python", "from", "import");
blockly.Blocks["indexer_Python"] = {
  init() {
    const value$$20 = this.appendValueInput("INDEX").appendField(new blockly.FieldVariable("{dictVariable}"), "VAR").appendField("[");
    value$$20, null;
    const value$$21 = this.appendDummyInput().appendField("]");
    value$$21, null;
    this.setInputsInline(true);
    this.setOutput(true);
    this.setColour(230);
    this.setTooltip("Gets an item from the variable at a given index. Not supported for all variables.");
    this.setHelpUrl("https://docs.python.org/3/reference/datamodel.html#object.__getitem__");
  }

};

blockly.Python["indexer_Python"] = function (block$$6) {
  var value$$22;
  const varName = blockly.Python.variableDB_.getName((value$$22 = block$$6.getFieldValue("VAR"), (String(value$$22))), blockly.Variables.NAME_TYPE);
  const input$$2 = blockly.Python.valueToCode(block$$6, "INDEX", blockly.Python.ORDER_ATOMIC);
  const code$$7 = varName + "[" + input$$2 + "]";
  return [code$$7, blockly.Python.ORDER_ATOMIC];
};

export function makeFunctionBlock_Python(blockName$$2, label, outputType, tooltip, helpurl, functionStr) {
  blockly.Blocks[blockName$$2] = {
    init() {
      console.log(blockName$$2 + " init");
      const value$$23 = this.appendValueInput("x").setCheck(null).appendField(label);
      value$$23, null;
      this.setInputsInline(true);
      this.setOutput(true, outputType);
      this.setColour(230);
      this.setTooltip(tooltip);
      this.setHelpUrl(helpurl);
    }

  };

  blockly.Python[blockName$$2] = function (block$$7) {
    const args = blockly.Python.valueToCode(block$$7, "x", blockly.Python.ORDER_MEMBER);
    const cleanArgs = replace(args, "^\\[|\\]$", "");
    const code$$8 = functionStr + "(" + cleanArgs + ")";
    return [code$$8, blockly.Python.ORDER_FUNCTION_CALL];
  };
}
makeFunctionBlock_Python("reversedBlock_Python", "reversed", "None", "Create a reversed iterator to reverse a list or a tuple; wrap it in a new list or tuple.", "https://docs.python.org/3/library/functions.html#reversed", "reversed");
makeFunctionBlock_Python("tupleConstructorBlock_Python", "tuple", "None", "Create a tuple from a list, e.g. ['a','b'] becomes ('a','b')", "https://docs.python.org/3/library/stdtypes.html#tuple", "tuple");
makeFunctionBlock_Python("dictBlock_Python", "dict", "None", "Create a dictionary from a list of tuples, e.g. [('a',1),('b',2)...]", "https://docs.python.org/3/tutorial/datastructures.html#dictionaries", "dict");
makeFunctionBlock_Python("zipBlock_Python", "zip", "Array", "Zip together two or more lists", "https://docs.python.org/3.3/library/functions.html#zip", "zip");
makeFunctionBlock_Python("sortedBlock_Python", "as sorted", "Array", "Sort lists of stuff", "https://docs.python.org/3.3/library/functions.html#sorted", "sorted");
makeFunctionBlock_Python("setBlock_Python", "set", "Array", "Make a set with unique members of a list.", "https://docs.python.org/2/library/sets.html", "set");
makeFunctionBlock_Python("boolConversion_Python", "as bool", "Boolean", "Convert something to Boolean.", "https://docs.python.org/3/library/stdtypes.html#boolean-values", "bool");
makeFunctionBlock_Python("strConversion_Python", "as str", "String", "Convert something to String.", "https://docs.python.org/3/library/stdtypes.html#str", "str");
makeFunctionBlock_Python("floatConversion_Python", "as float", "Number", "Convert something to Float.", "https://docs.python.org/3/library/functions.html#float", "float");
makeFunctionBlock_Python("intConversion_Python", "as int", "Number", "Convert something to Int.", "https://docs.python.org/3/library/functions.html#int", "int");
makeFunctionBlock_Python("getInput_Python", "input", "String", "Present the given prompt to the user and wait for their typed input response.", "https://docs.python.org/3/library/functions.html#input", "input");
blockly.Blocks["tupleBlock_Python"] = {
  init() {
    const value$$24 = this.appendValueInput("FIRST").setCheck(null).appendField("(");
    value$$24, null;
    const value$$25 = this.appendValueInput("SECOND").setCheck(null).appendField(",");
    value$$25, null;
    const value$$26 = this.appendDummyInput().appendField(")");
    value$$26, null;
    this.setInputsInline(true);
    this.setOutput(true, null);
    this.setColour(230);
    this.setTooltip("Use this to create a two-element tuple");
    this.setHelpUrl("https://docs.python.org/3/tutorial/datastructures.html#tuples-and-sequences");
  }

};

blockly.Python["tupleBlock_Python"] = function (block$$8) {
  const firstArg = blockly.Python.valueToCode(block$$8, "FIRST", blockly.Python.ORDER_ATOMIC);
  const secondArg = blockly.Python.valueToCode(block$$8, "SECOND", blockly.Python.ORDER_ATOMIC);
  const code$$9 = "(" + firstArg + "," + secondArg + ")";
  return [code$$9, blockly.Python.ORDER_NONE];
};

export const IntellisenseEntry = declare(function PythonToolbox_IntellisenseEntry(arg1, arg2, arg3, arg4) {
  this.Name = arg1;
  this.Info = arg2;
  this.isFunction = arg3;
  this.isClass = arg4;
}, Record);
export function IntellisenseEntry$reflection() {
  return record("PythonToolbox.IntellisenseEntry", [], IntellisenseEntry, () => [["Name", string], ["Info", string], ["isFunction", bool], ["isClass", bool]]);
}
export const IntellisenseVariable = declare(function PythonToolbox_IntellisenseVariable(arg1, arg2) {
  this.VariableEntry = arg1;
  this.ChildEntries = arg2;
}, Record);
export function IntellisenseVariable$reflection() {
  return record("PythonToolbox.IntellisenseVariable", [], IntellisenseVariable, () => [["VariableEntry", IntellisenseEntry$reflection()], ["ChildEntries", array$$6(IntellisenseEntry$reflection())]]);
}
export const notebooks = createAtom(null);
export function GetKernel() {
  if (!equals(notebooks(), null)) {
    const matchValue = notebooks().currentWidget;

    if (matchValue == null) {
      return null;
    } else {
      const widget = matchValue;
      const matchValue$$1 = widget.session.kernel;

      if (matchValue$$1 == null) {
        return null;
      } else {
        const kernel = matchValue$$1;
        return [widget, kernel];
      }
    }
  } else {
    return null;
  }
}
export function GetKernelCompletion(queryString) {
  const matchValue$$2 = GetKernel();

  if (matchValue$$2 == null) {
    return reject("no kernel");
  } else {
    const kernel$$1 = matchValue$$2[1];
    return PromiseBuilder$0024$0024Run$0024$0024212F1D4B(promise, PromiseBuilder$0024$0024Delay$0024$002462FBFDE1(promise, function () {
      return PromiseBuilder$0024$0024Delay$0024$002462FBFDE1(promise, function () {
        return kernel$$1.requestComplete(anonRecord({
          code: queryString,
          cursor_pos: queryString.length
        })).then(function (_arg1) {
          const content = _arg1.content;
          return Promise.resolve(content.matches.slice());
        });
      }).catch(function (_arg2) {
        return Promise.resolve([queryString + " is unavailable"]);
      });
    }));
  }
}
export function requestInspectTimeout(queryString$$1) {
  return new Promise(function (ok, er) {
    const value$$27 = setTimeout(function () {
      er((() => {
        throw new Error("timeout:" + queryString$$1);
      })());
    }, 100) | 0;
    value$$27, null;
  });
}
export function GetKernalInspection(queryString$$2) {
  const matchValue$$3 = GetKernel();

  if (matchValue$$3 == null) {
    console.log("NOKERNEL");
    return reject("no kernel");
  } else {
    const widget$$1 = matchValue$$3[0];
    const kernel$$2 = matchValue$$3[1];
    return PromiseBuilder$0024$0024Run$0024$0024212F1D4B(promise, PromiseBuilder$0024$0024Delay$0024$002462FBFDE1(promise, function () {
      return PromiseBuilder$0024$0024Delay$0024$002462FBFDE1(promise, function () {
        return kernel$$2.requestInspect(anonRecord({
          code: queryString$$2,
          cursor_pos: queryString$$2.length,
          detail_level: 0
        })).then(function (_arg1$$1) {
          const content$$1 = _arg1$$1.content;

          if (content$$1.found) {
            const mimeType = widget$$1.content.rendermime.preferredMimeType(content$$1.data);
            const renderer = widget$$1.content.rendermime.createRenderer(mimeType);
            const payload = content$$1.data;
            const model = new rendermime.MimeModel(anonRecord({
              data: payload
            }));
            return renderer.renderModel(model).then(function () {
              return Promise.resolve(renderer.node.innerText);
            });
          } else {
            console.log(queryString$$2 + ":UNDEFINED");
            return Promise.resolve("UNDEFINED");
          }
        });
      }).catch(function (_arg3) {
        console.log(queryString$$2 + ":UNAVAILABLE");
        return Promise.resolve(queryString$$2 + " is unavailable");
      });
    }));
  }
}
export const intellisenseLookup = new Map([]);
export function isFunction_Python(info) {
  return info.indexOf("Type: function") >= 0;
}
export function isClass_Python(info$$1) {
  return info$$1.indexOf("Type: class") >= 0;
}
export function RequestIntellisenseVariable_Python(block$$9, parentName) {
  let pr$$1;
  pr$$1 = PromiseBuilder$0024$0024Run$0024$0024212F1D4B(promise, PromiseBuilder$0024$0024Delay$0024$002462FBFDE1(promise, function () {
    return PromiseBuilder$0024$0024Delay$0024$002462FBFDE1(promise, function () {
      return GetKernalInspection(parentName).then(function (_arg1$$2) {
        const parent = new IntellisenseEntry(parentName, _arg1$$2, isFunction_Python(_arg1$$2), isClass_Python(_arg1$$2));
        let shouldGetChildren;
        const matchValue$$4 = tryGetValue(intellisenseLookup, parent.Name, null);
        shouldGetChildren = matchValue$$4[0] ? (matchValue$$4[1].VariableEntry.Info !== parent.Info ? true : matchValue$$4[1].ChildEntries.length === 0) ? true : false : true;
        return (shouldGetChildren ? GetKernelCompletion(parentName + ".").then(function (_arg2$$2) {
          var pr;
          let safeCompletions;
          safeCompletions = _arg2$$2.filter(function predicate(s) {
            var value$$28;

            if (parent.Info.indexOf("Signature: DataFrame") === 0) {
              if (value$$28 = s.indexOf("_") === 0, (!value$$28)) {
                const value$$29 = s.indexOf("style") === 0;
                return !value$$29;
              } else {
                return false;
              }
            } else {
              return true;
            }
          });
          return (pr = (map(function mapping(completion) {
            return GetKernalInspection(parentName + "." + completion);
          }, safeCompletions, Array)), (Promise.all(pr))).then(function (_arg3$$1) {
            let children;
            const array$$2 = zip(safeCompletions, _arg3$$1);
            children = map(function mapping$$1(tupledArg) {
              return new IntellisenseEntry(tupledArg[0], tupledArg[1], isFunction_Python(tupledArg[1]), isClass_Python(tupledArg[1]));
            }, array$$2, Array);
            const intellisenseVariable = new IntellisenseVariable(parent, children);

            if (intellisenseLookup.has(parentName)) {
              intellisenseLookup.set(parentName, intellisenseVariable);
              return Promise.resolve();
            } else {
              addToDict(intellisenseLookup, parentName, intellisenseVariable);
              return Promise.resolve();
            }
          });
        }) : (console.log("Not refreshing intellisense for " + parent.Name), Promise.resolve())).then(() => PromiseBuilder$0024$0024Delay$0024$002462FBFDE1(promise, function () {
          const intellisenseUpdateEvent = new Events.Change(block$$9, "field", "VAR", 0, 1);
          intellisenseUpdateEvent.group = "INTELLISENSE";
          console.log("event status is " + Events.disabled_);
          Events.disabled_ = 0;
          Events.fire(intellisenseUpdateEvent);
          return Promise.resolve();
        }));
      });
    }).catch(function (_arg4) {
      console.log("Intellisense event failed to fire; " + _arg4.message);
      return Promise.resolve();
    });
  }));
  pr$$1.then();
}
export function requestAndStubOptions_Python(block$$10, varName$$1) {
  var value$$30;

  if (varName$$1 !== "" ? (value$$30 = block$$10.isInFlyout, (!value$$30)) : false) {
    RequestIntellisenseVariable_Python(block$$10, varName$$1);
  }

  if (block$$10.isInFlyout) {
    return [[" ", " "]];
  } else if (varName$$1 !== "" ? (intellisenseLookup.has(varName$$1)) : false) {
    return [["!Waiting for kernel to respond with options.", "!Waiting for kernel to respond with options."]];
  } else {
    return [["!Not defined until you execute code.", "!Not defined until you execute code."]];
  }
}
export function getIntellisenseMemberOptions_Python(memberSelectionFunction, varName$$2) {
  let matchValue$$5;
  matchValue$$5 = tryGetValue(intellisenseLookup, varName$$2, null);
  var $target$$30;

  if (matchValue$$5[0]) {
    if (!matchValue$$5[1].VariableEntry.isFunction ? matchValue$$5[1].ChildEntries.length > 0 : false) {
      $target$$30 = 0;
    } else {
      $target$$30 = 1;
    }
  } else {
    $target$$30 = 1;
  }

  switch ($target$$30) {
    case 0:
      {
        let array$$4;
        array$$4 = matchValue$$5[1].ChildEntries.filter(memberSelectionFunction);
        return map(function mapping$$2(ie) {
          return [ie.Name, ie.Name];
        }, array$$4, Array);
      }

    case 1:
      {
        if (matchValue$$5[0]) {
          if (matchValue$$5[1].VariableEntry.Info === "UNDEFINED") {
            return [["!Not defined until you execute code.", "!Not defined until you execute code."]];
          } else {
            return [["!No properties available.", "!No properties available."]];
          }
        } else {
          return [["!Not defined until you execute code.", "!Not defined until you execute code."]];
        }
      }
  }
}
export function getIntellisenseVarTooltip(varName$$3) {
  let matchValue$$6;
  matchValue$$6 = tryGetValue(intellisenseLookup, varName$$3, null);

  if (matchValue$$6[0]) {
    return matchValue$$6[1].VariableEntry.Info;
  } else {
    return "!Not defined until you execute code.";
  }
}
export function getIntellisenseMemberTooltip(varName$$4, memberName) {
  let matchValue$$7;
  matchValue$$7 = tryGetValue(intellisenseLookup, varName$$4, null);

  if (matchValue$$7[0]) {
    let matchValue$$8;
    matchValue$$8 = tryFind(function predicate$$1(c) {
      return c.Name === memberName;
    }, matchValue$$7[1].ChildEntries);

    if (matchValue$$8 == null) {
      return "!Not defined until you execute code.";
    } else {
      const child = matchValue$$8;
      return child.Info;
    }
  } else {
    return "!Not defined until you execute code.";
  }
}
export function UpdateAllIntellisense_Python() {
  const workspace = blockly.getMainWorkspace();
  const blocks = workspace.getBlocksByType("varGetProperty_Python", false);
  addRangeInPlace(workspace.getBlocksByType("varDoMethod_Python", false), blocks);
  iterate(function (b) {
    b.updateIntellisense(b, null, function (varName$$5) {
      return requestAndStubOptions_Python(b, varName$$5);
    });
  }, blocks);
}
export function SafeRemoveField(block$$11, fieldName, inputName) {
  const matchValue$$9 = [block$$11.getField(fieldName), block$$11.getInput(inputName)];

  if (equals(matchValue$$9[0], null)) {} else if (equals(matchValue$$9[1], null)) {
    console.log("error removing (" + fieldName + ") from block; input (" + inputName + ") does not exist");
  } else {
    matchValue$$9[1].removeField(fieldName);
  }
}
export function makeMemberIntellisenseBlock_Python(blockName$$3, preposition, verb, memberSelectionFunction$$1, hasArgs, hasDot) {
  blockly.Blocks[blockName$$3] = {
    varSelectionUserName(thisBlockClosure, selectedOption) {
      const fieldVariable = thisBlockClosure.getField("VAR");
      let lastVar;
      const source = thisBlockClosure.workspace.getAllVariables();
      lastVar = last(source);
      let dataString;
      const value$$31 = thisBlockClosure.data;
      dataString = String(value$$31);
      const data = dataString.indexOf(":") >= 0 ? dataString.split(":") : [""];
      let selectionUserName;

      if (selectedOption == null) {
        const matchValue$$10 = [fieldVariable.getText(), data[0], lastVar.name];
        selectionUserName = matchValue$$10[0] === "" ? matchValue$$10[1] === "" ? matchValue$$10[2] : matchValue$$10[1] : matchValue$$10[0];
      } else {
        const option = selectedOption;
        let source$$2;
        const source$$1 = fieldVariable.getOptions();
        source$$2 = find(function predicate$$2(o) {
          return o[1] === option;
        }, source$$1);
        selectionUserName = head(source$$2);
      }

      return selectionUserName;
    },

    selectedMember: "",

    updateIntellisense(thisBlockClosure$$1, selectedVarOption, optionsFunction) {
      const input$$4 = thisBlockClosure$$1.getInput("INPUT");
      SafeRemoveField(thisBlockClosure$$1, "MEMBER", "INPUT");
      SafeRemoveField(thisBlockClosure$$1, "USING", "INPUT");
      const varUserName = thisBlockClosure$$1.varSelectionUserName(thisBlockClosure$$1, selectedVarOption);
      const options = optionsFunction(varUserName);
      const value$$33 = input$$4.appendField(new blockly.FieldDropdown(options, function (newMemberSelection) {
        var matchValue$$11;
        const thisFieldDropdown = this;
        thisFieldDropdown.setTooltip(getIntellisenseMemberTooltip(varUserName, newMemberSelection));
        thisBlockClosure$$1.selectedMember = (matchValue$$11 = [newMemberSelection.indexOf("!") === 0, this.selectedMember], matchValue$$11[1] === "" ? newMemberSelection : matchValue$$11[0] ? this.selectedMember : newMemberSelection);

        if (varUserName !== "") {
          thisBlockClosure$$1.data = varUserName + ":" + thisBlockClosure$$1.selectedMember;
        }

        return newMemberSelection;
      }), "MEMBER");
      value$$33, null;

      if (equals(thisBlockClosure$$1.data, null)) {
        thisBlockClosure$$1.data = varUserName + ":" + thisBlockClosure$$1.selectedMember;
      }

      const memberField = thisBlockClosure$$1.getField("MEMBER");
      memberField.setTooltip(getIntellisenseMemberTooltip(varUserName, memberField.getText()));

      if (hasArgs) {
        const value$$34 = input$$4.appendField("using", "USING");
        value$$34, null;
        thisBlockClosure$$1.setInputsInline(true);
      }
    },

    init() {
      console.log(blockName$$3 + " init");
      const thisBlockClosure$$2 = this;
      const input$$5 = hasArgs ? this.appendValueInput("INPUT") : this.appendDummyInput("INPUT");
      const value$$36 = input$$5.appendField(preposition).appendField(new blockly.FieldVariable("variable name", function (newSelection) {
        const thisFieldVariable = this;
        thisBlockClosure$$2.updateIntellisense(thisBlockClosure$$2, newSelection, function (varName$$6) {
          return requestAndStubOptions_Python(thisBlockClosure$$2, varName$$6);
        });
        return newSelection;
      }), "VAR").appendField(verb);
      value$$36, null;
      thisBlockClosure$$2.updateIntellisense(thisBlockClosure$$2, null, function (varName$$7) {
        return requestAndStubOptions_Python(thisBlockClosure$$2, varName$$7);
      });

      if (hasArgs) {
        this.setInputsInline(true);
      }

      this.setOutput(true);
      this.setColour(230);
      this.setTooltip("!Not defined until you execute code.");
      this.setHelpUrl("");
    },

    onchange(e$$1) {
      var value$$37, value$$38;

      if ((!equals(this.workspace, null) ? (value$$37 = this.isInFlyout, (!value$$37)) : false) ? e$$1.group === "INTELLISENSE" : false) {
        const data$$1 = (value$$38 = this.data, (String(value$$38))).split(":");
        this.updateIntellisense(this, null, function (varName$$8) {
          return getIntellisenseMemberOptions_Python(memberSelectionFunction$$1, varName$$8);
        });
        const memberField$$1 = this.getField("MEMBER");

        if (data$$1[1] !== "") {
          memberField$$1.setValue(data$$1[1]);
        }

        const varName$$9 = this.varSelectionUserName(this, null);
        this.setTooltip((getIntellisenseVarTooltip(varName$$9)));
      }
    }

  };

  blockly.Python[blockName$$3] = function (block$$12) {
    var value$$39;
    const varName$$11 = blockly.Python.variableDB_.getName((value$$39 = block$$12.getFieldValue("VAR"), (String(value$$39))), blockly.Variables.NAME_TYPE);
    let memberName$$1;
    const value$$40 = block$$12.getFieldValue("MEMBER");
    memberName$$1 = String(value$$40);
    let code$$10;

    if (memberName$$1.indexOf("!") === 0) {
      code$$10 = "";
    } else if (hasArgs) {
      const args$$1 = blockly.Python.valueToCode(block$$12, "INPUT", blockly.Python.ORDER_MEMBER);
      const cleanArgs$$1 = replace(args$$1, "^\\[|\\]$", "");
      code$$10 = varName$$11 + (hasDot ? "." : "") + memberName$$1 + "(" + cleanArgs$$1 + ")";
    } else {
      code$$10 = varName$$11 + (hasDot ? "." : "") + memberName$$1;
    }

    return [code$$10, blockly.Python.ORDER_FUNCTION_CALL];
  };
}
makeMemberIntellisenseBlock_Python("varGetProperty_Python", "from", "get", function (ie$$1) {
  return !ie$$1.isFunction;
}, false, true);
makeMemberIntellisenseBlock_Python("varDoMethod_Python", "with", "do", function (ie$$2) {
  return ie$$2.isFunction;
}, true, true);
makeMemberIntellisenseBlock_Python("varCreateObject_Python", "with", "create", function (ie$$3) {
  return ie$$3.isClass;
}, true, true);
export const FlyoutRegistryEntry = declare(function PythonToolbox_FlyoutRegistryEntry(arg1, arg2, arg3) {
  this.LanguageName = arg1;
  this.KernelCheckFunction = arg2;
  this.FlyoutFunction = arg3;
}, Record);
export function FlyoutRegistryEntry$reflection() {
  return record("PythonToolbox.FlyoutRegistryEntry", [], FlyoutRegistryEntry, () => [["LanguageName", string], ["KernelCheckFunction", lambda(string, bool)], ["FlyoutFunction", lambda(type("Blockly.Blockly.Workspace"), array$$6(type("Browser.Types.Element")))]]);
}

blockly.Variables.flyoutCategoryBlocks = function (workspace$$1) {
  if (!equals(blockly.Variables.flyoutRegistry, null)) {
    let registry$$1;
    const value$$41 = blockly.Variables.flyoutRegistry;
    registry$$1 = value$$41;
    const matchValue$$12 = GetKernel();

    if (matchValue$$12 != null) {
      const k = matchValue$$12[1];
      let entryOption;
      entryOption = tryFind$$1(function predicate$$3(e$$2) {
        return e$$2.KernelCheckFunction(k.name);
      }, registry$$1);

      if (entryOption != null) {
        const e$$3 = entryOption;
        return e$$3.FlyoutFunction(workspace$$1);
      } else {
        return [];
      }
    } else {
      return [];
    }
  } else {
    return [];
  }
};

export function flyoutCategoryBlocks_Python(workspace$$2) {
  const variableModelList = workspace$$2.getVariablesOfType("");
  const xmlList = [];

  if (0 < variableModelList.length) {
    const lastVarFieldXml = variableModelList[variableModelList.length - 1];

    if (blockly.Blocks.variables_set) {
      const xml$$2 = utils.xml.createElement("block");
      xml$$2.setAttribute("type", "variables_set");
      xml$$2.setAttribute("gap", blockly.Blocks.math_change ? "8" : "24");
      const value$$42 = xml$$2.appendChild(Variables.generateVariableFieldDom(lastVarFieldXml));
      value$$42, null;
      xmlList.push(xml$$2);
    }

    if (blockly.Blocks.math_change) {
      const xml$$3 = utils.xml.createElement("block");
      xml$$3.setAttribute("type", "math_change");
      xml$$3.setAttribute("gap", blockly.Blocks.math_change ? "20" : "8");
      const value$$43 = xml$$3.appendChild(Variables.generateVariableFieldDom(lastVarFieldXml));
      value$$43, null;
      const shadowBlockDom = Xml.textToDom("<value name='DELTA'><shadow type='math_number'><field name='NUM'>1</field></shadow></value>");
      const value$$44 = xml$$3.appendChild(shadowBlockDom);
      value$$44, null;
      xmlList.push(xml$$3);
    }

    if (blockly.Blocks.varGetProperty_Python) {
      const xml$$4 = utils.xml.createElement("block");
      xml$$4.setAttribute("type", "varGetProperty_Python");
      xml$$4.setAttribute("gap", blockly.Blocks.varGetPropertyPython ? "20" : "8");
      const value$$45 = xml$$4.appendChild(Variables.generateVariableFieldDom(lastVarFieldXml));
      value$$45, null;
      xmlList.push(xml$$4);
    }

    if (blockly.Blocks.varDoMethod_Python) {
      const xml$$5 = utils.xml.createElement("block");
      xml$$5.setAttribute("type", "varDoMethod_Python");
      xml$$5.setAttribute("gap", blockly.Blocks.varDoMethodPython ? "20" : "8");
      const value$$46 = xml$$5.appendChild(Variables.generateVariableFieldDom(lastVarFieldXml));
      value$$46, null;
      xmlList.push(xml$$5);
    }

    if (blockly.Blocks.varCreateObject_Python) {
      const xml$$6 = utils.xml.createElement("block");
      xml$$6.setAttribute("type", "varCreateObject_Python");
      xml$$6.setAttribute("gap", blockly.Blocks.varCreateObjectPython ? "20" : "8");
      const value$$47 = xml$$6.appendChild(Variables.generateVariableFieldDom(lastVarFieldXml));
      value$$47, null;
      xmlList.push(xml$$6);
    }

    if (blockly.Blocks.variables_get) {
      let sortedVars;

      const projection = function projection(v$$1) {
        return v$$1.name;
      };

      sortedVars = sortWith(function ($x$$1, $y$$2) {
        return comparePrimitives(projection($x$$1), projection($y$$2));
      }, variableModelList);
      iterate(function (variable) {
        const xml$$7 = utils.xml.createElement("block");
        xml$$7.setAttribute("type", "variables_get");
        xml$$7.setAttribute("gap", "8");
        const value$$48 = xml$$7.appendChild(Variables.generateVariableFieldDom(variable));
        value$$48, null;
        xmlList.push(xml$$7);
      }, sortedVars);
    }
  }

  return xmlList;
}
export const registry = equals(blockly.Variables.flyoutRegistry, null) ? [] : (() => {
  const value$$49 = blockly.Variables.flyoutRegistry;
  return value$$49;
})();
registry.push(new FlyoutRegistryEntry("Python", function (name$$1) {
  return name$$1.toLocaleLowerCase().indexOf("python") >= 0;
}, flyoutCategoryBlocks_Python));
blockly.Variables.flyoutRegistry = registry;
export const toolbox = "<xml xmlns=\"https://developers.google.com/blockly/xml\" id=\"toolbox\" style=\"display: none\">\n    <category name=\"IMPORT\" colour=\"255\">\n      <block type=\"importAs_Python\"></block>\n      <block type=\"importFrom_Python\"></block>\n    </category>\n    <category name=\"FREESTYLE\" colour=\"290\">\n      <block type=\"dummyOutputCodeBlock_Python\"></block>\n      <block type=\"dummyNoOutputCodeBlock_Python\"></block>\n      <block type=\"valueOutputCodeBlock_Python\"></block>\n      <block type=\"valueNoOutputCodeBlock_Python\"></block>\n    </category>\n    <category name=\"LOGIC\" colour=\"%{BKY_LOGIC_HUE}\">\n      <block type=\"controls_if\"></block>\n      <block type=\"logic_compare\"></block>\n      <block type=\"logic_operation\"></block>\n      <block type=\"logic_negate\"></block>\n      <block type=\"logic_boolean\"></block>\n      <block type=\"logic_null\"></block>\n      <block type=\"logic_ternary\"></block>\n    </category>\n    <category name=\"LOOPS\" colour=\"%{BKY_LOOPS_HUE}\">\n      <block type=\"controls_repeat_ext\">\n        <value name=\"TIMES\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">10</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"controls_whileUntil\"></block>\n      <block type=\"controls_for\">\n        <value name=\"FROM\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n        <value name=\"TO\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">10</field>\n          </shadow>\n        </value>\n        <value name=\"BY\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"comprehensionForEach_Python\"></block>\n      <block type=\"controls_forEach\"></block>\n      <block type=\"controls_flow_statements\"></block>\n    </category>\n    <category name=\"MATH\" colour=\"%{BKY_MATH_HUE}\">\n      <block type=\"math_number\">\n        <field name=\"NUM\">123</field>\n      </block>\n      <block type=\"math_arithmetic\">\n        <value name=\"A\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n        <value name=\"B\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_single\">\n        <value name=\"NUM\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">9</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_trig\">\n        <value name=\"NUM\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">45</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_constant\"></block>\n      <block type=\"math_number_property\">\n        <value name=\"NUMBER_TO_CHECK\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">0</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_round\">\n        <value name=\"NUM\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">3.1</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_on_list\"></block>\n      <block type=\"math_modulo\">\n        <value name=\"DIVIDEND\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">64</field>\n          </shadow>\n        </value>\n        <value name=\"DIVISOR\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">10</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_constrain\">\n        <value name=\"VALUE\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">50</field>\n          </shadow>\n        </value>\n        <value name=\"LOW\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n        <value name=\"HIGH\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">100</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_random_int\">\n        <value name=\"FROM\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n        <value name=\"TO\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">100</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"math_random_float\"></block>\n      <block type=\"math_atan2\">\n        <value name=\"X\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n        <value name=\"Y\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">1</field>\n          </shadow>\n        </value>\n      </block>\n    </category>\n    <category name=\"TEXT\" colour=\"%{BKY_TEXTS_HUE}\">\n      <block type=\"text\"></block>\n      <block type=\"text_join\"></block>\n      <block type=\"text_append\">\n        <value name=\"TEXT\">\n          <shadow type=\"text\"></shadow>\n        </value>\n      </block>\n      <block type=\"text_length\">\n        <value name=\"VALUE\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">abc</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"text_isEmpty\">\n        <value name=\"VALUE\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\"></field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"text_indexOf\">\n        <value name=\"VALUE\">\n          <block type=\"variables_get\">\n            <field name=\"VAR\">{textVariable}</field>\n          </block>\n        </value>\n        <value name=\"FIND\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">abc</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"text_charAt\">\n        <value name=\"VALUE\">\n          <block type=\"variables_get\">\n            <field name=\"VAR\">{textVariable}</field>\n          </block>\n        </value>\n      </block>\n      <block type=\"text_getSubstring\">\n        <value name=\"STRING\">\n          <block type=\"variables_get\">\n            <field name=\"VAR\">{textVariable}</field>\n          </block>\n        </value>\n      </block>\n      <block type=\"text_changeCase\">\n        <value name=\"TEXT\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">abc</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"text_trim\">\n        <value name=\"TEXT\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">abc</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"text_print\">\n        <value name=\"TEXT\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">abc</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"text_prompt_ext\">\n        <value name=\"TEXT\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">abc</field>\n          </shadow>\n        </value>\n      </block>\n      <!-- <block type=\"getInput\">\n        <value name=\"x\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">The prompt shown to the user</field>\n          </shadow>\n        </value>\n      </block> -->\n    </category>\n    <category name=\"LISTS\" colour=\"%{BKY_LISTS_HUE}\">\n      <block type=\"lists_create_with\">\n        <mutation items=\"0\"></mutation>\n      </block>\n      <block type=\"lists_create_with\"></block>\n      <block type=\"lists_repeat\">\n        <value name=\"NUM\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">5</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"lists_length\"></block>\n      <block type=\"lists_isEmpty\"></block>\n      <block type=\"lists_indexOf\">\n        <value name=\"VALUE\">\n          <block type=\"variables_get\">\n            <field name=\"VAR\">{listVariable}</field>\n          </block>\n        </value>\n      </block>\n      <block type=\"lists_getIndex\">\n        <value name=\"VALUE\">\n          <block type=\"variables_get\">\n            <field name=\"VAR\">{listVariable}</field>\n          </block>\n        </value>\n      </block>\n      <block type=\"lists_setIndex\">\n        <value name=\"LIST\">\n          <block type=\"variables_get\">\n            <field name=\"VAR\">{listVariable}</field>\n          </block>\n        </value>\n      </block>\n      <block type=\"lists_getSublist\">\n        <value name=\"LIST\">\n          <block type=\"variables_get\">\n            <field name=\"VAR\">{listVariable}</field>\n          </block>\n        </value>\n      </block>\n      <block type=\"indexer_Python\"></block>\n      <block type=\"lists_split\">\n        <value name=\"DELIM\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">,</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"lists_sort\"></block>\n      <block type=\"setBlock_Python\"></block>\n      <block type=\"sortedBlock_Python\"></block>\n      <block type=\"zipBlock_Python\"></block>\n      <block type=\"dictBlock_Python\"></block>\n      <block type=\"tupleBlock_Python\"></block>\n      <block type=\"tupleConstructorBlock_Python\"></block>\n      <block type=\"reversedBlock_Python\"></block>\n    </category>\n    <category name=\"COLOUR\" colour=\"%{BKY_COLOUR_HUE}\">\n      <block type=\"colour_picker\"></block>\n      <block type=\"colour_random\"></block>\n      <block type=\"colour_rgb\">\n        <value name=\"RED\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">100</field>\n          </shadow>\n        </value>\n        <value name=\"GREEN\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">50</field>\n          </shadow>\n        </value>\n        <value name=\"BLUE\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">0</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"colour_blend\">\n        <value name=\"COLOUR1\">\n          <shadow type=\"colour_picker\">\n            <field name=\"COLOUR\">#ff0000</field>\n          </shadow>\n        </value>\n        <value name=\"COLOUR2\">\n          <shadow type=\"colour_picker\">\n            <field name=\"COLOUR\">#3333ff</field>\n          </shadow>\n        </value>\n        <value name=\"RATIO\">\n          <shadow type=\"math_number\">\n            <field name=\"NUM\">0.5</field>\n          </shadow>\n        </value>\n      </block>Conversion\n    </category>\n    <category name=\"CONVERSION\" colour=\"120\">\n      <block type=\"boolConversion_Python\">\n      </block>\n      <block type=\"intConversion_Python\">\n      </block>\n      <block type=\"floatConversion_Python\">\n      </block>\n      <block type=\"strConversion_Python\">\n      </block>\n    </category>\n    <category name=\"I/O\" colour=\"190\">\n      <block type=\"withAs_Python\">\n      </block>\n      <block type=\"textFromFile_Python\">\n        <value name=\"FILENAME\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">name of file</field>\n          </shadow>\n        </value>\n      </block>\n      <block type=\"readFile_Python\">\n              <value name=\"FILENAME\">\n          <shadow type=\"text\">\n            <field name=\"TEXT\">name of file</field>\n          </shadow>\n        </value>\n      </block>\n    </category>\n    <sep></sep>\n    <category name=\"VARIABLES\" colour=\"%{BKY_VARIABLES_HUE}\" custom=\"VARIABLE\"></category>\n    <category name=\"FUNCTIONS\" colour=\"%{BKY_PROCEDURES_HUE}\" custom=\"PROCEDURE\"></category>\n  </xml>";